"""MCP Server for Directory Browser"""

import asyncio
import json
from pathlib import Path
from typing import Any, Sequence

from mcp.server.models import InitializationOptions
from mcp.server import NotificationOptions, Server
from mcp.server.stdio import stdio_server
from mcp.types import (
    CallToolRequest,
    ListToolsRequest,
    Tool,
    TextContent,
    ImageContent,
    EmbeddedResource,
)

from .directory_ops import DirectoryOperations
from .security import SecurityManager


class DirectoryBrowserServer:
    """目录浏览MCP服务器"""

    def __init__(self, config_path: str = None):
        self.server = Server("directory-browser")
        self.directory_ops = DirectoryOperations()
        self.security_manager = SecurityManager(config_path)

        # 注册工具
        self._register_tools()

    def _register_tools(self):
        """注册MCP工具"""

        @self.server.list_tools()
        async def handle_list_tools() -> list[Tool]:
            """列出可用的工具"""
            return [
                Tool(
                    name="list_directory",
                    description="列出目录中的文件和子目录",
                    inputSchema={
                        "type": "object",
                        "properties": {
                            "path": {
                                "type": "string",
                                "description": "要列出的目录路径"
                            },
                            "show_hidden": {
                                "type": "boolean",
                                "description": "是否显示隐藏文件（以.开头的文件）",
                                "default": False
                            }
                        },
                        "required": ["path"]
                    }
                ),
                Tool(
                    name="get_file_info",
                    description="获取文件或目录的详细信息",
                    inputSchema={
                        "type": "object",
                        "properties": {
                            "path": {
                                "type": "string",
                                "description": "文件或目录的路径"
                            }
                        },
                        "required": ["path"]
                    }
                ),
                Tool(
                    name="check_path_access",
                    description="检查路径的访问权限",
                    inputSchema={
                        "type": "object",
                        "properties": {
                            "path": {
                                "type": "string",
                                "description": "要检查的路径"
                            }
                        },
                        "required": ["path"]
                    }
                ),
                Tool(
                    name="open_directory",
                    description="在系统文件管理器中打开目录",
                    inputSchema={
                        "type": "object",
                        "properties": {
                            "path": {
                                "type": "string",
                                "description": "要打开的目录路径"
                            }
                        },
                        "required": ["path"]
                    }
                )
            ]

        @self.server.call_tool()
        async def handle_call_tool(name: str, arguments: dict) -> list[TextContent]:
            """处理工具调用"""
            try:
                if name == "list_directory":
                    return await self._handle_list_directory(arguments)
                elif name == "get_file_info":
                    return await self._handle_get_file_info(arguments)
                elif name == "check_path_access":
                    return await self._handle_check_path_access(arguments)
                elif name == "open_directory":
                    return await self._handle_open_directory(arguments)
                else:
                    return [TextContent(
                        type="text",
                        text=f"未知工具: {name}"
                    )]

            except Exception as e:
                return [TextContent(
                    type="text",
                    text=f"工具执行出错: {str(e)}"
                )]

    async def _handle_list_directory(self, arguments: dict) -> list[TextContent]:
        """处理列出目录请求"""
        path = arguments.get("path")
        show_hidden = arguments.get("show_hidden", False)

        if not path:
            return [TextContent(
                type="text",
                text="错误: 未提供路径参数"
            )]

        # 安全检查
        validation = self.security_manager.validate_request(path, "list")
        if not validation["valid"]:
            return [TextContent(
                type="text",
                text=f"访问被拒绝: {validation['error']}"
            )]

        # 执行目录列表操作
        result = self.directory_ops.list_directory(
            validation["path"],
            show_hidden
        )

        if "error" in result:
            return [TextContent(
                type="text",
                text=f"列出目录失败: {result['error']}"
            )]

        # 格式化输出
        output = self._format_directory_listing(result)

        return [TextContent(
            type="text",
            text=output
        )]

    async def _handle_get_file_info(self, arguments: dict) -> list[TextContent]:
        """处理获取文件信息请求"""
        path = arguments.get("path")

        if not path:
            return [TextContent(
                type="text",
                text="错误: 未提供路径参数"
            )]

        # 安全检查
        validation = self.security_manager.validate_request(path, "info")
        if not validation["valid"]:
            return [TextContent(
                type="text",
                text=f"访问被拒绝: {validation['error']}"
            )]

        # 获取文件信息
        result = self.directory_ops.get_file_info(validation["path"])

        if "error" in result:
            return [TextContent(
                type="text",
                text=f"获取文件信息失败: {result['error']}"
            )]

        # 格式化输出
        output = self._format_file_info(result)

        return [TextContent(
            type="text",
            text=output
        )]

    async def _handle_check_path_access(self, arguments: dict) -> list[TextContent]:
        """处理检查路径访问权限请求"""
        path = arguments.get("path")

        if not path:
            return [TextContent(
                type="text",
                text="错误: 未提供路径参数"
            )]

        # 执行访问检查
        result = self.directory_ops.check_path_access(path)
        security_check = self.security_manager.is_path_allowed(path)

        # 合并结果
        combined_result = {
            **result,
            "security_allowed": security_check["allowed"],
            "security_reason": security_check.get("reason", "")
        }

        if "error" in result:
            return [TextContent(
                type="text",
                text=f"检查访问权限失败: {result['error']}"
            )]

        # 格式化输出
        output = self._format_access_info(combined_result)

        return [TextContent(
            type="text",
            text=output
        )]

    async def _handle_open_directory(self, arguments: dict) -> list[TextContent]:
        """处理打开目录请求"""
        path = arguments.get("path")

        if not path:
            return [TextContent(
                type="text",
                text="错误: 未提供路径参数"
            )]

        # 安全检查
        validation = self.security_manager.validate_request(path, "open")
        if not validation["valid"]:
            return [TextContent(
                type="text",
                text=f"访问被拒绝: {validation['error']}"
            )]

        # 执行打开目录操作
        result = self.directory_ops.open_directory(validation["path"])

        if "error" in result:
            return [TextContent(
                type="text",
                text=f"打开目录失败: {result['error']}"
            )]

        return [TextContent(
            type="text",
            text=result["message"]
        )]

    def _format_directory_listing(self, result: dict) -> str:
        """格式化目录列表输出"""
        lines = [
            f"目录: {result['path']}",
            f"总计: {result['total_count']} 项",
            "-" * 60
        ]

        for entry in result["entries"]:
            type_icon = "📁" if entry["type"] == "directory" else "📄"
            size_str = f"{entry['size']:,} bytes" if entry["type"] == "file" else ""

            lines.append(
                f"{type_icon} {entry['name']:<30} {size_str:<15} {entry['modified']}"
            )

        return "\n".join(lines)

    def _format_file_info(self, info: dict) -> str:
        """格式化文件信息输出"""
        lines = [
            f"路径: {info['path']}",
            f"名称: {info['name']}",
            f"类型: {info['type']}",
            f"大小: {info['size']:,} bytes",
            f"修改时间: {info['modified']}",
        ]

        if "created" in info:
            lines.extend([
                f"创建时间: {info['created']}",
                f"访问时间: {info['accessed']}",
                f"权限: {info['permissions']}",
                f"可读: {'是' if info['readable'] else '否'}",
                f"可写: {'是' if info['writable'] else '否'}",
                f"可执行: {'是' if info['executable'] else '否'}",
            ])

            if info["type"] == "file" and "extension" in info:
                lines.append(f"扩展名: {info['extension']}")
            elif info["type"] == "directory" and "item_count" in info:
                lines.append(f"包含项目: {info['item_count']}")

        return "\n".join(lines)

    def _format_access_info(self, info: dict) -> str:
        """格式化访问权限信息输出"""
        lines = [
            f"路径: {info['path']}",
            f"存在: {'是' if info['exists'] else '否'}",
            f"安全检查: {'通过' if info['security_allowed'] else '失败'}",
        ]

        if not info["security_allowed"]:
            lines.append(f"安全原因: {info['security_reason']}")

        if info["exists"]:
            lines.extend([
                f"类型: {info.get('type', '未知')}",
                f"可读: {'是' if info['readable'] else '否'}",
                f"可写: {'是' if info['writable'] else '否'}",
                f"可执行: {'是' if info['executable'] else '否'}",
            ])

        return "\n".join(lines)

    async def run(self):
        """运行MCP服务器"""
        async with stdio_server() as (read_stream, write_stream):
            await self.server.run(
                read_stream,
                write_stream,
                InitializationOptions(
                    server_name="directory-browser",
                    server_version="0.1.0",
                    capabilities=self.server.get_capabilities(
                        notification_options=NotificationOptions(),
                        experimental_capabilities={},
                    ),
                ),
            )


async def main():
    """主函数"""
    # 查找配置文件
    config_path = None
    possible_configs = [
        "config.json",
        Path(__file__).parent.parent.parent / "config.json",
        Path.home() / ".directory_browser_mcp" / "config.json"
    ]

    for config_file in possible_configs:
        if Path(config_file).exists():
            config_path = str(config_file)
            break

    # 创建并运行服务器
    server = DirectoryBrowserServer(config_path)
    await server.run()


if __name__ == "__main__":
    asyncio.run(main())


"""Security module for access control"""

import json
import os
from pathlib import Path
from typing import List, Dict, Any, Optional


class SecurityManager:
    """安全管理器，负责访问控制和路径验证"""

    def __init__(self, config_path: Optional[str] = None):
        """
        初始化安全管理器

        Args:
            config_path: 配置文件路径
        """
        self.allowed_directories: List[str] = []
        self.show_hidden_files: bool = False
        self.max_entries: int = 1000

        if config_path:
            self.load_config(config_path)

    def load_config(self, config_path: str) -> bool:
        """
        加载配置文件

        Args:
            config_path: 配置文件路径

        Returns:
            是否加载成功
        """
        try:
            with open(config_path, 'r', encoding='utf-8') as f:
                config = json.load(f)

            self.allowed_directories = [
                str(Path(d).resolve()) for d in config.get('allowed_directories', [])
            ]
            self.show_hidden_files = config.get('show_hidden_files', False)
            self.max_entries = config.get('max_entries', 1000)

            return True

        except (FileNotFoundError, json.JSONDecodeError, Exception) as e:
            print(f"加载配置文件失败: {e}")
            # 使用默认配置
            self.allowed_directories = [str(Path.home())]
            return False

    def is_path_allowed(self, path: str) -> Dict[str, Any]:
        """
        检查路径是否被允许访问

        Args:
            path: 要检查的路径

        Returns:
            包含检查结果的字典
        """
        try:
            # 规范化路径
            normalized_path = str(Path(path).resolve())

            # 检查路径穿越攻击
            if self._has_path_traversal(path):
                return {
                    "allowed": False,
                    "reason": "检测到路径穿越攻击",
                    "path": normalized_path
                }

            # 检查是否在允许的目录中
            for allowed_dir in self.allowed_directories:
                if normalized_path.startswith(allowed_dir):
                    return {
                        "allowed": True,
                        "path": normalized_path,
                        "allowed_root": allowed_dir
                    }

            return {
                "allowed": False,
                "reason": "路径不在允许的目录范围内",
                "path": normalized_path,
                "allowed_directories": self.allowed_directories
            }

        except Exception as e:
            return {
                "allowed": False,
                "reason": f"路径验证失败: {str(e)}",
                "path": path
            }

    def validate_request(self, path: str, operation: str) -> Dict[str, Any]:
        """
        验证请求的安全性

        Args:
            path: 请求的路径
            operation: 操作类型 (list, info, check, open)

        Returns:
            验证结果
        """
        # 检查路径权限
        path_check = self.is_path_allowed(path)
        if not path_check["allowed"]:
            return {
                "valid": False,
                "error": path_check["reason"],
                "details": path_check
            }

        # 检查操作权限
        if operation not in ["list", "info", "check", "open"]:
            return {
                "valid": False,
                "error": f"不支持的操作: {operation}"
            }

        # 检查路径是否存在且可访问
        try:
            target_path = Path(path).resolve()
            if not target_path.exists():
                return {
                    "valid": False,
                    "error": f"路径不存在: {path}"
                }

            if not os.access(target_path, os.R_OK):
                return {
                    "valid": False,
                    "error": f"没有读取权限: {path}"
                }

            return {
                "valid": True,
                "path": str(target_path),
                "operation": operation
            }

        except Exception as e:
            return {
                "valid": False,
                "error": f"路径验证失败: {str(e)}"
            }

    def _has_path_traversal(self, path: str) -> bool:
        """
        检查路径是否包含路径穿越攻击

        Args:
            path: 要检查的路径

        Returns:
            是否包含路径穿越
        """
        # 检查常见的路径穿越模式
        dangerous_patterns = [
            '../',
            '..\\',
            '..',
            '//',
            '\\\\',
            '~/',
            '${'
        ]

        normalized_path = path.replace('\\', '/').lower()

        for pattern in dangerous_patterns:
            if pattern in normalized_path:
                return True

        return False

    def get_allowed_directories(self) -> List[str]:
        """
        获取允许访问的目录列表

        Returns:
            允许访问的目录列表
        """
        return self.allowed_directories.copy()

    def add_allowed_directory(self, directory: str) -> bool:
        """
        添加允许访问的目录

        Args:
            directory: 目录路径

        Returns:
            是否添加成功
        """
        try:
            normalized_dir = str(Path(directory).resolve())
            if normalized_dir not in self.allowed_directories:
                self.allowed_directories.append(normalized_dir)
                return True
            return False
        except Exception:
            return False

    def remove_allowed_directory(self, directory: str) -> bool:
        """
        移除允许访问的目录

        Args:
            directory: 目录路径

        Returns:
            是否移除成功
        """
        try:
            normalized_dir = str(Path(directory).resolve())
            if normalized_dir in self.allowed_directories:
                self.allowed_directories.remove(normalized_dir)
                return True
            return False
        except Exception:
            return False

    def get_config_summary(self) -> Dict[str, Any]:
        """
        获取配置摘要

        Returns:
            配置信息摘要
        """
        return {
            "allowed_directories": self.allowed_directories,
            "show_hidden_files": self.show_hidden_files,
            "max_entries": self.max_entries,
            "total_allowed_dirs": len(self.allowed_directories)
        }


"""Entry point for directory browser MCP server"""

import asyncio
from .server import main as async_main


def main():
    """同步入口点函数"""
    asyncio.run(async_main())


if __name__ == "__main__":
    main()


"""Directory operations module for MCP server"""

import os
import stat
import subprocess
import platform
from pathlib import Path
from datetime import datetime
from typing import List, Dict, Any, Optional


class DirectoryOperations:
    """Core directory and file operations"""

    def __init__(self):
        pass

    def list_directory(self, path: str, show_hidden: bool = False) -> Dict[str, Any]:
        """
        列出目录内容

        Args:
            path: 目录路径
            show_hidden: 是否显示隐藏文件

        Returns:
            包含目录内容信息的字典
        """
        try:
            target_path = Path(path).resolve()

            if not target_path.exists():
                return {"error": f"路径不存在: {path}"}

            if not target_path.is_dir():
                return {"error": f"不是目录: {path}"}

            entries = []

            for entry in target_path.iterdir():
                # 跳过隐藏文件（如果不显示）
                if not show_hidden and entry.name.startswith('.'):
                    continue

                try:
                    entry_info = self._get_entry_info(entry)
                    entries.append(entry_info)
                except (OSError, PermissionError):
                    # 跳过无法访问的文件
                    continue

            # 按类型和名称排序（目录在前）
            entries.sort(key=lambda x: (x['type'] != 'directory', x['name'].lower()))

            return {
                "path": str(target_path),
                "entries": entries,
                "total_count": len(entries)
            }

        except PermissionError:
            return {"error": f"没有访问权限: {path}"}
        except Exception as e:
            return {"error": f"列出目录失败: {str(e)}"}

    def get_file_info(self, path: str) -> Dict[str, Any]:
        """
        获取文件或目录的详细信息

        Args:
            path: 文件或目录路径

        Returns:
            包含文件信息的字典
        """
        try:
            target_path = Path(path).resolve()

            if not target_path.exists():
                return {"error": f"路径不存在: {path}"}

            info = self._get_entry_info(target_path, detailed=True)
            return info

        except PermissionError:
            return {"error": f"没有访问权限: {path}"}
        except Exception as e:
            return {"error": f"获取文件信息失败: {str(e)}"}

    def check_path_access(self, path: str) -> Dict[str, Any]:
        """
        检查路径访问权限

        Args:
            path: 路径

        Returns:
            包含访问权限信息的字典
        """
        try:
            target_path = Path(path).resolve()

            result = {
                "path": str(target_path),
                "exists": target_path.exists(),
                "readable": False,
                "writable": False,
                "executable": False
            }

            if target_path.exists():
                result["readable"] = os.access(target_path, os.R_OK)
                result["writable"] = os.access(target_path, os.W_OK)
                result["executable"] = os.access(target_path, os.X_OK)
                result["type"] = "directory" if target_path.is_dir() else "file"

            return result

        except Exception as e:
            return {"error": f"检查访问权限失败: {str(e)}"}

    def _get_entry_info(self, path: Path, detailed: bool = False) -> Dict[str, Any]:
        """获取文件或目录的基本信息"""
        try:
            stat_info = path.stat()

            entry_info = {
                "name": path.name,
                "path": str(path),
                "type": "directory" if path.is_dir() else "file",
                "size": stat_info.st_size,
                "modified": datetime.fromtimestamp(stat_info.st_mtime).isoformat()
            }

            if detailed:
                entry_info.update({
                    "created": datetime.fromtimestamp(stat_info.st_ctime).isoformat(),
                    "accessed": datetime.fromtimestamp(stat_info.st_atime).isoformat(),
                    "permissions": oct(stat_info.st_mode)[-3:],
                    "owner_uid": stat_info.st_uid,
                    "group_gid": stat_info.st_gid,
                    "readable": os.access(path, os.R_OK),
                    "writable": os.access(path, os.W_OK),
                    "executable": os.access(path, os.X_OK)
                })

                if path.is_file():
                    entry_info["extension"] = path.suffix.lower()
                elif path.is_dir():
                    try:
                        # 统计目录中的项目数量
                        item_count = len(list(path.iterdir()))
                        entry_info["item_count"] = item_count
                    except PermissionError:
                        entry_info["item_count"] = "无法访问"

            return entry_info

        except Exception as e:
            return {
                "name": path.name,
                "path": str(path),
                "error": f"获取信息失败: {str(e)}"
            }

    def open_directory(self, path: str) -> Dict[str, Any]:
        """
        打开目录（在系统文件管理器中）

        Args:
            path: 目录路径

        Returns:
            包含操作结果的字典
        """
        try:
            target_path = Path(path).resolve()

            if not target_path.exists():
                return {"error": f"路径不存在: {path}"}

            if not target_path.is_dir():
                return {"error": f"不是目录: {path}"}

            # 根据操作系统选择合适的命令
            system = platform.system().lower()

            if system == "darwin":  # macOS
                subprocess.run(["open", str(target_path)], check=True)
            elif system == "windows":  # Windows
                subprocess.run(["explorer", str(target_path)], check=True)
            elif system == "linux":  # Linux
                # 尝试多种可能的文件管理器
                file_managers = ["xdg-open", "nautilus", "thunar", "dolphin", "pcmanfm"]
                for fm in file_managers:
                    try:
                        subprocess.run([fm, str(target_path)], check=True)
                        break
                    except (subprocess.CalledProcessError, FileNotFoundError):
                        continue
                else:
                    return {"error": "未找到可用的文件管理器"}
            else:
                return {"error": f"不支持的操作系统: {system}"}

            return {
                "success": True,
                "path": str(target_path),
                "message": f"已在系统文件管理器中打开目录: {target_path}"
            }

        except subprocess.CalledProcessError as e:
            return {"error": f"打开目录失败: {str(e)}"}
        except PermissionError:
            return {"error": f"没有访问权限: {path}"}
        except Exception as e:
            return {"error": f"打开目录失败: {str(e)}"}

    def _format_size(self, size_bytes: int) -> str:
        """格式化文件大小"""
        for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
            if size_bytes < 1024.0:
                return f"{size_bytes:.1f} {unit}"
            size_bytes /= 1024.0
        return f"{size_bytes:.1f} PB"